// www.elegoo.com
// www.settorezero.com

#include <Servo.h>  //servo library
Servo myservo; // create servo object to control servo

// Ultrasonic sensor pins
int Echo = A4;  
int Trig = A5; 

// default speed
#define defaultSpeed 140
// actual speed
int carSpeed=defaultSpeed;

//Line Tracking IO define
#define LT_R !digitalRead(10)
#define LT_M !digitalRead(4)
#define LT_L !digitalRead(2)

// IO used by motor control
#define ENA 5
#define ENB 6
#define IN1 7
#define IN2 8
#define IN3 9
#define IN4 11

// flag used to stop the car
bool canRun=false;

// after x milli-second I can reactivate obstacle control
#define waitObstacle 2000
long prevObstacleStopTime=0;

//Ultrasonic distance measurement Sub function
int distanceTest() 
  {
  digitalWrite(Trig, LOW);   
  delayMicroseconds(2);
  digitalWrite(Trig, HIGH);  
  delayMicroseconds(20);
  digitalWrite(Trig, LOW);   
  float Fdistance = pulseIn(Echo, HIGH);  
  Fdistance= Fdistance / 58;       
  return (int)Fdistance;
  }  

void forward()
  {
  analogWrite(ENA, carSpeed);
  analogWrite(ENB, carSpeed);
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, HIGH);
  }

void back()
  {
  analogWrite(ENA, carSpeed);
  analogWrite(ENB, carSpeed);
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, HIGH);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  }

void left()
  {
  analogWrite(ENA, carSpeed);
  analogWrite(ENB, carSpeed);
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, HIGH);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, HIGH);
  }

void right()
  {
  analogWrite(ENA, carSpeed);
  analogWrite(ENB, carSpeed);
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW); 
  } 

void stop()
  {
  digitalWrite(ENA, LOW);
  digitalWrite(ENB, LOW);
  } 

void setup()
  {
  Serial.begin(9600);
  myservo.attach(3);  // attach servo on pin 3 to servo object
  pinMode(Echo, INPUT);    
  pinMode(Trig, OUTPUT);  
  pinMode(LT_R,INPUT);
  pinMode(LT_M,INPUT);
  pinMode(LT_L,INPUT);
  stop();
  canRun=false;
  myservo.write(90);              
  delay(1000);
  }

void loop() 
  {
  static bool lastObstacle=false;
  char getstr = Serial.read();
  switch(getstr)
    {
    case 's': 
      canRun=!canRun;
      canRun?Serial.println("Start!"):Serial.println("Manual stop");  
      break;

    default:  
      break;
    } 

  // detect if there is a frontal obstacle
  if (distanceTest()<20 && (millis()-prevObstacleStopTime>waitObstacle))
    {
      // lastObstacle flag is only used for printing sentence only once
      // if you omit this, you'll get "Obstacle!" on serial continuously
      // until an obstacle is detected, and this is bad!
      if (!lastObstacle)
        {
        lastObstacle=true;
        Serial.println("Obstacle!");        
        }
    canRun=false;  
    prevObstacleStopTime=millis();
    }
  else
    {
    lastObstacle=false;  
    }
  
  
  // car can Run?
  if (canRun)
    {
    if(LT_M)
      {
      carSpeed=defaultSpeed;
      forward();
      }
    else if(LT_R) 
      { 
      carSpeed=defaultSpeed;
      right();
      while(LT_R);                             
      }   
    else if(LT_L) 
      {
      carSpeed=defaultSpeed;
      left();
      while(LT_L);  
      }
    } // \canRun
  else
    {
    stop();
    }
  } // \loop
